package xyz.lychee.lagfixer.libs.adventure.inventory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import xyz.lychee.lagfixer.libs.adventure.builder.AbstractBuilder;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.util.Buildable;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Unmodifiable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;
import xyz.lychee.lagfixer.libs.examination.Examinable;

@NonExtendable
public interface Book extends Buildable<Book, Builder>, Examinable {
   static @NotNull Book book(final @NotNull Component title, final @NotNull Component author, final @NotNull Collection<Component> pages) {
      return new BookImpl(title, author, new ArrayList(pages));
   }

   static @NotNull Book book(final @NotNull Component title, final @NotNull Component author, final @NotNull Component... pages) {
      return book(title, author, (Collection)Arrays.asList(pages));
   }

   @NotNull
   static @NotNull Builder builder() {
      return new BookImpl.BuilderImpl();
   }

   @NotNull Component title();

   @Contract(
      value = "_ -> new",
      pure = true
   )
   @NotNull Book title(final @NotNull Component title);

   @NotNull Component author();

   @Contract(
      value = "_ -> new",
      pure = true
   )
   @NotNull Book author(final @NotNull Component author);

   @Unmodifiable @NotNull List<Component> pages();

   @Contract(
      value = "_ -> new",
      pure = true
   )
   default @NotNull Book pages(final @NotNull Component... pages) {
      return this.pages(Arrays.asList(pages));
   }

   @Contract(
      value = "_ -> new",
      pure = true
   )
   @NotNull Book pages(final @NotNull List<Component> pages);

   @NotNull
   default @NotNull Builder toBuilder() {
      return builder().title(this.title()).author(this.author()).pages((Collection)this.pages());
   }

   public interface Builder extends AbstractBuilder<Book>, Buildable.Builder<Book> {
      @Contract("_ -> this")
      @NotNull
      @NotNull Builder title(final @NotNull Component title);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder author(final @NotNull Component author);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder addPage(final @NotNull Component page);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder pages(final @NotNull Component... pages);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder pages(final @NotNull Collection<Component> pages);

      @NotNull Book build();
   }
}
